type bool =
| True
| False

type 'a option =
| Some of 'a
| None

type ('a, 'b) prod =
| Pair of 'a * 'b

type 'a list =
| Nil
| Cons of 'a * 'a list

type sumbool =
| Left
| Right

(** val bool_dec : bool -> bool -> sumbool **)

let bool_dec b1 b2 =
  match b1 with
  | True ->
    (match b2 with
     | True -> Left
     | False -> Right)
  | False ->
    (match b2 with
     | True -> Right
     | False -> Left)

(** val in_dec : ('a1 -> 'a1 -> sumbool) -> 'a1 -> 'a1 list -> sumbool **)

let rec in_dec h a = function
| Nil -> Right
| Cons (y, l0) ->
  let s = h y a in
  (match s with
   | Left -> Left
   | Right -> in_dec h a l0)

type ascii =
| Ascii of bool * bool * bool * bool * bool * bool * bool * bool

(** val ascii_dec : ascii -> ascii -> sumbool **)

let ascii_dec a b =
  let Ascii (x, x0, x1, x2, x3, x4, x5, x6) = a in
  let Ascii (b8, b9, b10, b11, b12, b13, b14, b15) = b in
  (match bool_dec x b8 with
   | Left ->
     (match bool_dec x0 b9 with
      | Left ->
        (match bool_dec x1 b10 with
         | Left ->
           (match bool_dec x2 b11 with
            | Left ->
              (match bool_dec x3 b12 with
               | Left ->
                 (match bool_dec x4 b13 with
                  | Left ->
                    (match bool_dec x5 b14 with
                     | Left -> bool_dec x6 b15
                     | Right -> Right)
                  | Right -> Right)
               | Right -> Right)
            | Right -> Right)
         | Right -> Right)
      | Right -> Right)
   | Right -> Right)

type string =
| EmptyString
| String of ascii * string

(** val string_dec : string -> string -> sumbool **)

let rec string_dec s s0 =
  match s with
  | EmptyString ->
    (match s0 with
     | EmptyString -> Left
     | String (a, s1) -> Right)
  | String (a, s1) ->
    (match s0 with
     | EmptyString -> Right
     | String (a0, s2) ->
       (match ascii_dec a a0 with
        | Left -> string_dec s1 s2
        | Right -> Right))

type guard =
  string
  (* singleton inductive, whose constructor was Gu *)

type transition_label =
| Transition_label of string * (string, string) prod list

type transition =
| Local of string * string * transition_label * guard * bool
| From_sub of string * string * string * transition_label * guard * bool
| To_sub of string * string * string * transition_label * guard * bool

type astd =
| Elem of string
| Automata of string * listOfAstdStates * transition list * string list
   * string list * string
and listOfAstdStates =
| EmptyAstdStates
| ConsAstdStates of astd * listOfAstdStates

type operation_label =
| Label of string * (string, string) prod list

type predicate =
| Equality of string * string
| And of predicate * predicate
| Or of predicate * predicate
| TruePredicate
| FalsePredicate
| IsIn of string * string list
| Guard of string

type substitution =
| Select of selectList
| Parallel of substitution * substitution
| Affectation of string * string
| SkipSubstitution
and selectList =
| EmptySelectList
| ConsSelectList of predicate * substitution * selectList

type operation =
| Operation of operation_label * predicate * substitution

(** val get_name : astd -> string **)

let get_name = function
| Elem name -> name
| Automata (name, l, l0, l1, l2, s) -> name

(** val finalPredicate : astd -> predicate **)

let rec finalPredicate = function
| Elem e -> TruePredicate
| Automata (name, states, transitions, sFinal, dFinal, init) ->
  (match dFinal with
   | Nil ->
     (match sFinal with
      | Nil -> FalsePredicate
      | Cons (s, l) -> IsIn (name, sFinal))
   | Cons (he, ta) ->
     let dFinalPred = makeDFinalPred dFinal states name in
     (match sFinal with
      | Nil -> dFinalPred
      | Cons (s, l) -> Or ((IsIn (name, sFinal)), dFinalPred)))

(** val makeDFinalPred :
    string list -> listOfAstdStates -> string -> predicate **)

and makeDFinalPred dFinal states name =
  match states with
  | EmptyAstdStates -> FalsePredicate
  | ConsAstdStates (he, ta) ->
    let finalPredQueue = makeDFinalPred dFinal ta name in
    let nameOfHead = get_name he in
    (match in_dec string_dec nameOfHead dFinal with
     | Left ->
       Or ((And ((Equality (name, nameOfHead)), (finalPredicate he))),
         finalPredQueue)
     | Right -> finalPredQueue)

(** val getAstdFromStateList : string -> listOfAstdStates -> astd option **)

let rec getAstdFromStateList name = function
| EmptyAstdStates -> None
| ConsAstdStates (he, ta) ->
  (match string_dec (get_name he) name with
   | Left -> Some he
   | Right -> getAstdFromStateList name ta)

(** val initializeAllVariable : astd -> substitution **)

let rec initializeAllVariable = function
| Elem e -> SkipSubstitution
| Automata (name, states, l, l0, l1, init) ->
  Parallel ((Affectation (name, init)), (initializeFromAstdList states))

(** val initializeFromAstdList : listOfAstdStates -> substitution **)

and initializeFromAstdList = function
| EmptyAstdStates -> SkipSubstitution
| ConsAstdStates (headAstd, queueAstdList) ->
  Parallel ((initializeAllVariable headAstd),
    (initializeFromAstdList queueAstdList))

(** val getGuard : guard -> string **)

let getGuard gu =
  gu

(** val makePrecondition :
    transition -> string -> listOfAstdStates -> predicate option **)

let makePrecondition transition0 nameOfTranslatedAstd listOfStates =
  match transition0 with
  | Local (fromState, toState, astdTransition, astdPredicate, finale) ->
    let cAll = Equality (nameOfTranslatedAstd, fromState) in
    let cGuard = Guard (getGuard astdPredicate) in
    (match getAstdFromStateList fromState listOfStates with
     | Some fromAstd ->
       (match finale with
        | True ->
          let cFinal = finalPredicate fromAstd in
          Some (And ((And (cAll, cGuard)), cFinal))
        | False -> Some (And (cAll, cGuard)))
     | None -> None)
  | From_sub (fromState, toState, throughState, astdTransition,
              astdPredicate, finale) ->
    let cAll = Equality (nameOfTranslatedAstd, throughState) in
    let cGuard = Guard (getGuard astdPredicate) in
    let cToSub = Equality (throughState, fromState) in
    (match getAstdFromStateList throughState listOfStates with
     | Some throughAstd ->
       (match throughAstd with
        | Elem s -> None
        | Automata (nameThroughState, listOfSubStates, l, l0, l1, s) ->
          (match getAstdFromStateList fromState listOfSubStates with
           | Some fromAstd ->
             (match finale with
              | True ->
                let cFinale = finalPredicate fromAstd in
                Some (And ((And ((And (cAll, cGuard)), cToSub)), cFinale))
              | False -> Some (And ((And (cAll, cGuard)), cToSub)))
           | None -> None))
     | None -> None)
  | To_sub (fromState, toState, throughState, astdTransition, astdPredicate,
            finale) ->
    let cAll = Equality (nameOfTranslatedAstd, fromState) in
    let cGuard = Guard (getGuard astdPredicate) in
    (match getAstdFromStateList fromState listOfStates with
     | Some fromAstd ->
       (match finale with
        | True ->
          let cFinal = finalPredicate fromAstd in
          Some (And ((And (cAll, cGuard)), cFinal))
        | False -> Some (And (cAll, cGuard)))
     | None -> None)

(** val makePostcondition :
    transition -> string -> listOfAstdStates -> substitution option **)

let makePostcondition transition0 nameOfTranslatedAstd listOfStates =
  match transition0 with
  | Local (fromState, toState, sigma, gu, finale) ->
    let tAll = Affectation (nameOfTranslatedAstd, toState) in
    (match getAstdFromStateList toState listOfStates with
     | Some a ->
       let tInitN2 = initializeAllVariable a in
       Some (Parallel (tAll, tInitN2))
     | None -> None)
  | From_sub (fromState, toState, throughState, sigma, gu, finale) ->
    let tAll = Affectation (nameOfTranslatedAstd, toState) in
    (match getAstdFromStateList toState listOfStates with
     | Some a ->
       let tInitN2 = initializeAllVariable a in
       Some (Parallel (tAll, tInitN2))
     | None -> None)
  | To_sub (fromState, toState, throughState, sigma, gu, final) ->
    let tAll = Affectation (nameOfTranslatedAstd, throughState) in
    let tToSub = Affectation (throughState, toState) in
    (match getAstdFromStateList throughState listOfStates with
     | Some a0 ->
       (match a0 with
        | Elem a -> None
        | Automata (nameThroughState, listThroughState, l, l0, l1, s) ->
          (match string_dec toState (String ((Ascii (False, False, False,
                   True, False, False, True, False)), (String ((Ascii (False,
                   True, False, True, False, True, False, False)),
                   EmptyString)))) with
           | Left -> Some tAll
           | Right ->
             (match string_dec toState (String ((Ascii (False, False, False,
                      True, False, False, True, False)), EmptyString)) with
              | Left ->
                let initHInit = initializeFromAstdList listThroughState in
                Some (Parallel (tAll, initHInit))
              | Right ->
                (match getAstdFromStateList toState listThroughState with
                 | Some a ->
                   let tInitFrom = initializeAllVariable a in
                   Some (Parallel ((Parallel (tAll, tToSub)), tInitFrom))
                 | None -> None))))
     | None -> None)

(** val getTransitionLabel : transition -> transition_label **)

let getTransitionLabel = function
| Local (s, s0, la, g, b) -> la
| From_sub (s, s0, s1, la, g, b) -> la
| To_sub (s, s0, s1, la, g, b) -> la

(** val transformTransitionLabel : transition_label -> operation_label **)

let transformTransitionLabel = function
| Transition_label (name, param) -> Label (name, param)

(** val paramAreEquals :
    (string, string) prod list -> (string, string) prod list -> bool **)

let rec paramAreEquals param1 param2 =
  match param1 with
  | Nil ->
    (match param2 with
     | Nil -> True
     | Cons (y, l) -> False)
  | Cons (y, qParam1) ->
    let Pair (p1, p1T) = y in
    (match param2 with
     | Nil -> False
     | Cons (y0, qParam2) ->
       let Pair (p2, p2T) = y0 in
       (match string_dec p1 p2 with
        | Left ->
          (match string_dec p1T p2T with
           | Left -> paramAreEquals qParam1 qParam2
           | Right -> False)
        | Right -> False))

(** val labelsAreEqual : operation_label -> transition_label -> bool **)

let labelsAreEqual label1 label2 =
  let Label (name1, param1) = label1 in
  let Transition_label (name2, param2) = label2 in
  (match string_dec name1 name2 with
   | Left -> paramAreEquals param1 param2
   | Right -> False)

(** val addTransition :
    transition -> operation list -> string -> listOfAstdStates -> operation
    list option **)

let rec addTransition tr li name s =
  let o = makePrecondition tr name s in
  let o0 = makePostcondition tr name s in
  (match o with
   | Some pre ->
     (match o0 with
      | Some post ->
        (match li with
         | Nil ->
           Some (Cons ((Operation
             ((transformTransitionLabel (getTransitionLabel tr)), pre,
             (Select (ConsSelectList (pre, post, EmptySelectList))))), Nil))
         | Cons (op, qLi) ->
           let Operation (label, preOp, postOp) = op in
           (match labelsAreEqual label (getTransitionLabel tr) with
            | True ->
              (match postOp with
               | Select li0 ->
                 Some (Cons ((Operation (label, (Or (pre, preOp)), (Select
                   (ConsSelectList (pre, post, li0))))), qLi))
               | _ -> None)
            | False ->
              (match addTransition tr qLi name s with
               | Some li0 -> Some (Cons (op, li0))
               | None -> None)))
      | None -> None)
   | None -> None)

(** val translateAllTransitions :
    transition list -> operation list -> string -> listOfAstdStates ->
    operation list option **)

let rec translateAllTransitions trans li name s =
  match trans with
  | Nil -> Some li
  | Cons (tr, qTrans) ->
    (match translateAllTransitions qTrans li name s with
     | Some trLi -> addTransition tr trLi name s
     | None -> None)

(** val opLabelsAreEqual : operation_label -> operation_label -> bool **)

let opLabelsAreEqual label1 label2 =
  let Label (name1, param1) = label1 in
  let Label (name2, param2) = label2 in
  (match string_dec name1 name2 with
   | Left -> paramAreEquals param1 param2
   | Right -> False)

(** val addOperation :
    operation -> operation list -> operation list option **)

let rec addOperation op opList =
  let Operation (label, pre, post) = op in
  (match opList with
   | Nil -> Some (Cons (op, Nil))
   | Cons (hOpList, qOpList) ->
     let Operation (labelHOp, preHOp, postHOp) = hOpList in
     (match opLabelsAreEqual label labelHOp with
      | True ->
        (match postHOp with
         | Select li ->
           Some (Cons ((Operation (label, (Or (pre, preHOp)), (Select
             (ConsSelectList (pre, post, li))))), qOpList))
         | _ -> None)
      | False ->
        (match addOperation op qOpList with
         | Some li -> Some (Cons (hOpList, li))
         | None -> None)))

(** val mergeOperationList :
    operation list -> operation list -> operation list option **)

let rec mergeOperationList aInB qLiInB =
  match aInB with
  | Nil -> Some qLiInB
  | Cons (op, qAInB) ->
    (match mergeOperationList qAInB qLiInB with
     | Some li -> addOperation op li
     | None -> None)

(** val modifyPrePost :
    operation list -> string -> string -> operation list **)

let rec modifyPrePost aInB nameOfA nameOfState =
  match aInB with
  | Nil -> Nil
  | Cons (op, qAInB) ->
    let Operation (label, pre, post) = op in
    Cons ((Operation (label, (And ((Equality (nameOfA, nameOfState)), pre)),
    (Select (ConsSelectList ((And ((Equality (nameOfA, nameOfState)), pre)),
    (Parallel ((Affectation (nameOfA, nameOfState)), post)),
    EmptySelectList))))), (modifyPrePost qAInB nameOfA nameOfState))

(** val translateAstd : astd -> operation list option **)

let rec translateAstd = function
| Elem s -> Some Nil
| Automata (name, states, trans, sFinal, dFinal, init) ->
  (match translateStateList states name with
   | Some li ->
     (match translateAllTransitions trans Nil name states with
      | Some opLi -> mergeOperationList opLi li
      | None -> None)
   | None -> None)

(** val translateStateList :
    listOfAstdStates -> string -> operation list option **)

and translateStateList li name =
  match li with
  | EmptyAstdStates -> Some Nil
  | ConsAstdStates (a, qLi) ->
    let o = translateAstd a in
    let o0 = translateStateList qLi name in
    (match o with
     | Some aInB ->
       (match o0 with
        | Some qLiInB ->
          mergeOperationList (modifyPrePost aInB name (get_name a)) qLiInB
        | None -> None)
     | None -> None)

